from .HitObjects.HitObject import HitObject

class Beatmap(object):
    Difficulty = {
        "HPDrainRate": None,
        "CircleSize": None,
        "OverallDifficulty": None,
        "ApproachRate": None,
        "SliderMultiplier": None,
        "SliderTickRate": None
    }

    HitObjects = []
    
    # public TimingPoints: Array<{
    #     Time: number,
    #     BeatLength: number,
    #     TimeSignature: number
    # }>;
    TimingPoints = []

    # public DifficultyTimingPoints: Array<{
    #     Time: number,
    #     SpeedMultiplier: number
    # }>;
    DifficultyTimingPoints = []

    Version = 0
    StackLeniency = 0

    def __init__(self):
        self.Difficulty = {
        "HPDrainRate": 0,
        "CircleSize": 0,
        "OverallDifficulty": 0,
        "ApproachRate": 0,
        "SliderMultiplier": 0,
        "SliderTickRate": 0
        }

        self.HitObjects = []
        self.TimingPoints = []
        self.DifficultyTimingPoints = []


from enum import Enum

class HitType(Enum):
    Normal     = 1 << 0,
    Slider     = 1 << 1,
    NewCombo   = 1 << 2,
    Spinner    = 1 << 3,
    ComboSkip1 = 1 << 4,
    ComboSkip2 = 1 << 5,
    ComboSkip3 = 1 << 6,
    Hold       = 1 << 7


from abc import ABC
from ...Vector2 import Vector2

class HitObject(ABC):
    Position = None
    StartTime = None
    Radius = None
    StackHeight = None
    StackedPosition = None

    def __init__(self, pos, startTime, **kwargs):
        radius = kwargs.get('radius', None)
        self.Position = pos
        self.StartTime = startTime
        self.Radius = radius

    def calculateStackedPosition(self, scale):
        coordinate = self.StackHeight * scale * -6.4
        stackOffset = Vector2(coordinate, coordinate)
        if(self.Position is not None):
            self.StackedPosition = self.Position.add(stackOffset)